


#' This script provides of few additional example of the use of the methods seen in the 
#' course LBOE2121. 
#' This examples have been executed "in live" during the lecture.
#' 
#' 




source("/home/gilles/stats/mytoolbox.R")



# /* 
# ---------------- Correlations visualisations -------------------
# */



#' Example 
# data(state.x77)
# d <- as.data.frame(state.x77)
# d$Population <- log10(d$Population)
# d$Area <- sqrt(d$Area)
# 
# corheatmap(d)
# corheatmap(d, R = FALSE)
# corheatmap(d, Rcex = 1.5, cexCol = 1, cexRow = 1)
# pairs2(d)
# pairs2((d))


d <- read.csv2("/home/gilles/stats/Formation_R_stats/UCL_LBOE2121/GBIF/UTM5data.csv")
summary(d)
colnames(d)


dev.new(width = 18/2.54, height = 28/2.54)
corheatmap(d[,-(1:10)], FALSE, cexRow = 0.6, cexCol = 0.7)


x11(18/2.54, 28/2.54)
d2 <- d[, which(colnames(d) == "bio01") : which(colnames(d) == "wind12")]
corheatmap(d2, R = FALSE, cexRow = 0.6, cexCol = 0.7)
corheatmap(d2, R = FALSE,  breaks = c(-1, -0.8,-0.6, 0.6, 0.8, 1), cexRow = 0.6, cexCol = 0.7)

x11(width = 14/2.54, height = 14/2.54)
d2 <- d[, which(colnames(d) == "bio01") : which(colnames(d) == "bio19")]
corheatmap(d2,T, 0.7)

x11(width = 14/2.54, height = 14/2.54)
d2 <- d[, which(colnames(d) == "clc111") : which(colnames(d) == "clc523")]
corheatmap(sqrt(d2),T, 0.5)

x11(width = 18/2.54, height = 14/2.54)

d2 <- d[, which(colnames(d) == "tmax01") : which(colnames(d) == "tmin12")]
corheatmap(d2,T, 0.5)

x11(width = 18/2.54, height = 14/2.54)

pairs2(cbind(d$alt, d2[,c( "tmin01", "tmin02", "tmin03", "tmin04",
                           "tmax04", "tmax05", "tmax06", "tmax07")]), reorder = FALSE)



# /* 
# ---------------- PCA - theory  -------------------
# */


#' PCA
#' 
#' 
#' Each variable of the data table is seen as a dimension in which you can position the observations (rows).
#' The PCA will create new variables (axes) that are a combination of the original axes (variables/columns).
#' The first of these new axes are build to capture as much variation as possible from the original variables
#' and these new axes are build to be uncorrelated.
#' 
#' The aims are 
#' 
#' * the obtention of these new uncorrelated axes (Principal Components) for further use
#' * the representation of the (eucildean) distances between the rows in 2 (or a few) dimensions
#' * the representation of the correlations or covariances between the columns/original variables
#' 
#' Questions and steps in a PCA analysis (heavily inspired by Legendre & Legendre 2012)
#' 
#' 1) Are the variables adapted to PCA ? 
#' 
#' You need a table with observations x variables.
#' You cannot use a contingency table (use Correspondance analysis for this kind of table).
#' 
#' * Ideally gaussian quantitative variables.
#' * Ordinal and binary are OK
#' * If quantitative variables too much skewed : transform (typically log, sqrt)
#' 
#' Typically 2 type of problems : 
#' a) qualitative variables --> use other distances measures (eg Gower) and PCoA or NMDS
#' b) if many 0 : double 0 will be considered as similarities
#' 
#' - this is not a problem with some type of measures (ex : precipitations)
#' - this is a problom with other type of measures (typically species abundances)
#' 
#' --> apply Hellinger transformations and then PCA. Tou work then on the relative abundance 
#' of each species on each site (so you loose the information of the absolute abundance).
#' --> use other adapted distances measures (Sorensen, Bray-Curtis) and then PCoA or NMDS
#' see Legendre & Legendre for guidance on the choice of the distance measure.
#' 
#' 
#' 2) Should we use standardised variables or not ? 
#' = Should we use the correlation or the covariance matrix ?
#' 
#' - if the units are not the same between descriptors --> you must use standardised variables
#' - if the dimensions are homogenous you can choose : 
#'      + if you want that all variables contribute equaly : use standardised variables
#'      + if you want that the variables that have higher variance contribute more : 
#'      use unstandardized variables. 
#' 
#'  With species data we often use unstandardized data as we want that dominant species
#'  have indeed a stronger effect on the ordianation than the very rare ones. 
#'  Correspondance analysis (and chi squared distance) on the contrary place more weight on the rare species
#' 
#' 
#' 3) How informative is the representation and which axes/eignevalues are important ?
#' --> screeplot (`eigenplot` function).
#' % of variance for each axis. 
#' Rough rule of thumb : Axes with eignevale > mean eigenvalue = important ones
#' 
#' 
#' 4) How to represent the results ? How to interpret these representations ?
#' 
#' If you want to compare the distances between the rows (sites) : distance biplots. 
#' The angles between the arrows are meaningless
#' 
#' If you want to visualize the correlations/covariances between the variables : 
#' correlation biplot or circle of correlations.
#' The angles represent the correlations/covariances but the distances between the sites are 
#' no more related to their euclidean distance.
#' 
#' In both cases you can project the observations/sites/rows on the axes/arrows
#' 
#' You can also use a separate graph for the sites and a correlation circle plot for 
#' the variables.
#' 
#' Always represent the variables with arrows to decrease the risk of misinterpretation.
#' The variables could also be represented as weighted averages of the sites in some 
#' circumstances but this is unusual with PCA (while it is the standard representation 
#' in Correspondance Analysis) 
#' 
#' 
#' 5) Quality of the representation of the variables in the reduced space/ contribution of
#' each variable to the reduced space : 
#' 
#' - correlation circle : `corplot`
#' - circle of equilibrium contribution on distance biplots: `biplot2`
#' - cos2 for the variables (% of their variance represneted in the reduced space) - 
#' visualize this with a heatmap : `cos2`, `cos2vars` and `cos2heatmap`
#' 
#' 
#' 6) Quality of the representation of the observations in the reduced space : 
#' 
#' - cos2 for the observations (use heatmap for visualization) : `cos2`, `cos2obs` and `cos2heatmap`
#' - Shepherd diagram to compare the distance in the reduced space with the original distance
#' 


# /* 
# ---------------- PCA - double 0  -------------------
# */

#' Illustration of the double 0 problem


#' 3 sites with 12 land cover variables : surfaces of diferent trees/forest composition.
#' Site b could be an area of open peatbogs and c could be a heavily urbanized area.
#' 
d <- rbind(a = rep (100, 12), b = 0, c = 0)
d[1:2,1] <- 100
d[c(1,3),12] <- 100

colnames(d) <- c("Picea", "Pseudotsuga", "Larix", "Pinus", "MixedCon", 
                 "MixedDec", "Quercus", "Fagus", "Fraxinus", "Carpinus", "Betula",
                 "Populus")
d

#' Site b is expected to be closer to site a than to site c. But for 
#' euclidean distance c and b are closer due to the shared 0

dist(d)

# graphical representation of the distances
mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

# PCA
biplot(rda(d), scaling = 1)


#' Same dataset but now the data represent the rainfall for each month of the year.
#' Here you expect that b is closer to c because the absence of rainfall during
#' 10 months of the year is realy a similarity.
#' 

d <- rbind(a = rep (100, 12), b = 0, c = 0)
d[1:2,1] <- 100
d[c(1,3),12] <- 100

colnames(d) <- paste0("prec", 1:12)
d

dist(d)

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))


#' this kind of problem is more critical with species composition data 
#' (and particularly when the sum of the rows are very different)
d <- rbind(
    a = c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1),
    b = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1),
    c = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1)
)

dist(d)

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))



biplot(rda(d), scaling = 1)
biplot(rda(decostand(d, "hellinger")), scaling = 1)



# /* 
# ---------------- PCA - example with Bioclim data  -------------------
# */


#' Example of PCA on the BIO worldclim variables
#' 
source("/home/gilles/stats/mytoolbox.R")

library(vegan)
library(FactoMineR)

d <- read.csv2("/home/gilles/stats/Formation_R_stats/UCL_LBOE2121/GBIF/UTM5data.csv")
summary(d)
colnames(d)



# PCA with vegan
d2 <- d[, which(colnames(d) == "bio01") : which(colnames(d) == "bio19")]
res <- rda(d2, scale = TRUE)

#' Screeplot
dev.new(width = 16/2.54, height = 6/2.54)
par(mar = c(4,3, 1, 1) , cex = 0.8)
eigenplot(res)

#' Default vegan biplot - wtf !!
dev.new(width = 15/2.54, height = 15/2.54)
biplot(res, scaling = 1)

#' Correlations circle plot
dev.new(width = 12/2.54, height = 12/2.54)
par(mar = c(3,3, 2, 1), mgp = c(1.8, 0.5, 0), cex = 1)
corplot(res)

#' Distance and correlation biplot (use labels = FALSE !!!)
dev.new(width = 20/2.54, height = 10/2.54)
par(mfrow = c(1,2), mar = c(3,3, 2, 1), mgp = c(1.8, 0.5, 0), cex = 0.8)
biplot2(res, sc = 0.2, scaling = 1, labels = FALSE)
biplot2(res, sc = 0.5, scaling = 2, labels = FALSE, var.col = "orangered")


#' Add colors for the different regions on the distance biplot 
#' to help the interpretation
#' 
mycols <- c("darkgreen", "gold", "lightgreen", "yellow", "khaki", 
            "chocolate4", "red", "blue", "black", "orangered")
dev.new(width = 16/2.54, height = 12/2.54)
par(mar = c(3,3, 2, 1), mgp = c(1.8, 0.5, 0), cex = 0.8)
biplot2(res, sc = 0.1, scaling = 1, labels = FALSE, 
        obs.col = mycols[d$naturalRegion], obs.cex = 1.5)
legend("bottomright", bty = "n", pch = 20, col = mycols, 
       legend = levels(d$naturalRegion), pt.cex = 2)


#' Cos2 of the variables
cos2(res)[[1]]
cos2heatmap(cos2(res)[[1]])

#' Cos2 of the variables
head(cos2(res)[[2]])
cos2heatmap(cos2(res)[[2]][,1:7], notecex = 0.001, cexRow = 0.1)

randomsample <- sample(1:nrow(d2), size = 200)
cos2heatmap(cos2(res)[[2]][randomsample,1:7], notecex = 0.001, cexRow = 0.1)



# /* 
# ---------------- Clustering - Theory -------------------
# */



d <- cbind(x = c(1, 2, 4, 8, 10), y = 0)
row.names(d) <- c(1, 2, 4, 8, 10)

x11()
plot(d)

D <- dist(d)
D

dev.new(width = 14/2.54, height = 20/2.54)
xhang = -1

par(mfrow = c(4,2), mar = c(0.1, 2.5, 2, 1), mgp = c(1.5, 0.5, 0))
# Single : shortest distance among groups
plot(hclust(D,method = "single"), hang = xhang , ylab ="", main = "Single linkage")
abline(h = seq(0,10,1), col = "gray70")

# Complete : longest distance among groups
plot(hclust(D,method = "complete"), hang = xhang , ylab ="", main = "Complete linkage" )
abline(h = seq(0,10,1), col = "gray70")

# Avergage : average distance among groups = UPGMA
plot(hclust(D,method = "average"), hang = xhang , ylab ="", main = "Average = UPGMA" )
abline(h = seq(0,10,1), col = "gray70")
mean(c(4, 7, 6, 6, 9, 8))

# Avergage : median distance among groups = WPGMA
plot(hclust(D,method = "median"), hang = xhang , ylab ="", main = "Median = WPGMA" )
abline(h = seq(0,10,1), col = "gray70")

# UPGMC
plot(hclust(D,method = "centroid"), hang = xhang , ylab ="", main = "Centroid = UPGMC" )
abline(h = seq(0,10,1), col = "gray70")

# WPGMC
plot(hclust(D,method = "mcquitty"), hang = xhang , ylab ="", main = "Mc Quitty = WPGMC" )
abline(h = seq(0,10,1), col = "gray70")

# Ward 1
plot(hclust(D,method = "ward.D"), hang = xhang , ylab ="", main = "Ward 1" )
abline(h = seq(0,10,1), col = "gray70")

# Ward 2
plot(hclust(D,method = "ward.D2"), hang = xhang , ylab ="", main = "Ward 2" )
abline(h = seq(0,10,1), col = "gray70")


# /* 
# ---------------- Clustering + Heatmap - Example with bioclim data-------------------
# */



source("/home/gilles/stats/mytoolbox.R")

library(vegan)
library(FactoMineR)
library(gplots)

d <- read.csv2("/home/gilles/stats/Formation_R_stats/UCL_LBOE2121/GBIF/UTM5data.csv")
summary(d)
colnames(d)

d2 <- d[, which(colnames(d) == "bio01") : which(colnames(d) == "bio19")]
d2 <- scale(d2) # on standardise la matrice


# Clustering pour les lignes : ward sur distance euclidienne de la matrice standardisée
cl <- hclust(dist(d2), method = "ward.D2")
rowcl <- as.dendrogram(reorder(cl,  rowMeans((d2))))

# Clustering pour les colonnes : ward sur les corrélations transformées en distances
corenv <- cor(d2)
cl <- hclust(as.dist(1-corenv), method = "ward.D2")
colcl <- as.dendrogram(reorder(cl,  rowMeans((1-corenv))))


# Bande de couleurs à ajouter dans la marge pour représenter la région naturelle
mycols <- c("darkgreen", "gold", "lightgreen", "yellow", "khaki", 
            "chocolate4", "red", "blue", "black", "orangered")
grcol <- mycols[d$naturalRegion]


#+ fig.width = 15/2.54, fig.height = 18/2.54
# dev.new(width = 18/2.54, height = 26/2.54)
heatmap.2(d2, Colv = colcl,  Rowv = rowcl, 
          col = colorRampPalette(c("dodgerblue", "white", "red"))(5), 
          cexRow = 0.01, cexCol = 1, 
          offsetCol = 0, offsetRow = 0, margins = c(3 , 2.5),
          trace = "none", density.info = "none",
          lhei = c(1,5), lwid = c(2,5), 
          key.par=list(mar = c(4,2,4,2), mgp = c(1, 0.5, 0)), 
          key.title = "", key.xlab =  "Standardized values",
          # add colored band
          RowSideColors = grcol
)

x11(width = 7/2.54, height = 3/2.54)
par(mar = c(0,0,0,0))
plot(x=1, type = "n", axes = TRUE, xlab = "", ylab = "", 
     xlim = c(0,1), ylim = c(0,1))
legend("right", inset = 0.2, fill = mycols, border = NA, legend = levels(d$naturalRegion), 
       bty = "n", ncol = 2, xpd = NA, x.intersp = 0.5, text.width = 0.3, cex = 1)

savepdf(file = "/home/gilles/Bureau/heatmap.pdf")
savepng(file = "/home/gilles/Bureau/heatmap.png", res = 600)



# /* 
# ---------------- Clustering + PCA - Example with bioclim data-------------------
# */

#' Dendrogramme sans heatmap 
#' 
#+ fig.width = 15/2.54, fig.height = 18/2.54
# dev.new(width = 26/2.54, height = 12/2.54)
cl <- hclust(dist(d2), method = "ward.D2")
cl <- reorder(cl,  rowMeans((d2)))
groups <- cutree(cl, k = 5)

par( mar = c(1,1,2,0), cex = 0.75)
coph <- coph_average <- cophenetic(cl)
Rcoph <- round(cor(dist(d2), coph), 3)
plot(cl, main = "", hang = -1, axes = FALSE, xlab = "", ylab = "", labels = FALSE)
title("Ward on Euclidean dist", line = 1)
mtext(paste0("Cophenetic Cor. = ", Rcoph), cex = 0.7, line = -0.25)
# symbols(x = 1:nrow(d2), y = rep(0, nrow(d2)), circles=rep(1, nrow(d2)),
#         bg= grcol, add=TRUE, inches=.02)
symbols(x = 1:nrow(d2), y = rep(-10, nrow(d2)), rectangles= cbind(rep(1, nrow(d2)), 16),
        bg= grcol[cl$order], fg = NA, add=TRUE, inches=FALSE)
k = 5
rect.hclust(cl, k = k)
# mtext(cutree(cl, k = k)[cl$order], at = 1:length(cl$order), side = 1, cex = 0.7, line = -0.25)




#' Add colors for the different groups defined by the dendrogram 
#' on a PCA
#' 
d2 <- d[, which(colnames(d) == "bio01") : which(colnames(d) == "bio19")]
d2 <- scale(d2) # on standardise la matrice
res <- rda(d2, scale = TRUE)


cl <- hclust(dist(d2), method = "ward.D2")
groups <- cutree(cl, k = 5)

mycols <- c("black", "darkblue", "skyblue", "gold", "orangered")
dev.new(width = 16/2.54, height = 12/2.54)
par(mar = c(3,3, 2, 1), mgp = c(1.8, 0.5, 0), cex = 0.8)
biplot2(res, sc = 0.1, scaling = 1, labels = FALSE, 
        obs.col = mycols[groups], obs.cex = 1)
legend("bottomright", bty = "n", pch = 20, col = mycols, 
       legend = paste("Group", 1:k), pt.cex = 2)


#' Add colors for the different regions on the distance biplot 
#' to help the interpretation
#' 
mycols <- c("darkgreen", "gold", "lightgreen", "yellow", "khaki", 
            "chocolate4", "red", "blue", "black", "orangered")
dev.new(width = 16/2.54, height = 12/2.54)
par(mar = c(3,3, 2, 1), mgp = c(1.8, 0.5, 0), cex = 0.8)
biplot2(res, sc = 0.1, scaling = 1, labels = FALSE, 
        obs.col = mycols[d$naturalRegion], obs.cex = 1)
legend("bottomright", bty = "n", pch = 20, col = mycols, 
       legend = levels(d$naturalRegion), pt.cex = 2)




# /* 
# ---------------- non additivity problem -------------------
# */



# create a dataset with non linear relationships due to interaction (non additivity)
set.seed(1)
x1 <- runif(100, 0, 10)
x2 <- runif(100, 0, 10)
x3 <- runif(100, 0, 10)
x4 <- runif(100, 0, 10)
x5 <- runif(100, 0, 10)

y <- 1500 + 100 * log(x1) +  x2 + 50* x3 - 20*x2*x3  + rnorm(100, 0, 20)
d <- round(data.frame(y, x1, x2, x3, x4, x5),1)




# y <- 1000 + 100 * log(x1) + x2 + x3 - 10*x2*x3  + rnorm(100, 0, 20)


m <- lm(y ~ x1 + x2 + x3 + x4  + x5 , data=d)
diagplot(m)
diagplot2(m)

m <- glm(y ~ log(x1) + x2 + x3 + x4  + x5 , data=d)
diagplot(m)
diagplot2(m)

m <- glm(I(y^2) ~ x1 + x2 + x3 + x4  + x5 , data=d)
diagplot(m)
diagplot2(m)


m <- glm(y ~ x1 + x2 * x3 + x4  + x5 , data=d)
diagplot(m)
diagplot2(m)


m <- glm(y ~ log(x1) + x2 * x3 + x4  + x5 , data=d)
diagplot(m)
diagplot2(m)



par(mfrow = c(2,3), mar = c(3,3,0.5,0.5), mgp = c(1.7, 0.6,0))
visreg(m, scale = 'response', partial = TRUE)

par(mfrow = c(2,3), mar = c(3,3,0.5,0.5), mgp = c(1.7, 0.6,0))

visreg(m, xvar = "x3", by = "x2", scale = 'response', partial = TRUE)
visreg(m, xvar = "x2", by = "x3", scale = 'response', partial = TRUE)



par(mfrow = c(1,1), mar = c(3,3,0.5,0.5), mgp = c(1.7, 0.6,0))
visreg2d(m, "x2", "x3",  scale=c("response"),
         plot.type=c("rgl"))
visreg2d(m, "x2", "x3",  scale=c("response"),
         plot.type=c("image"))




# /* 
# ---------------- Binomial model with non linearity -------------------
# */


# Simulation of a binomial dataset with non linearity problems


set.seed(12)
n <- 100
x1 <- runif(n, 0, 6)
x2 <- runif(n, 0, 10000)
x3 <- runif(n, 0, 10)
x4 <- runif(n, 0, 10)
bloc = factor(rep(LETTERS[1:5], n/5))

linpred <- 10 + 3 * x1 - 0.6 * x1^2 - 1.5 * log(x2)
y <- rbinom(n, p = plogis(linpred), size = 1)
# sum(y)
d <- round(data.frame(y, x1, x2, x3, x4),1)
d$bloc <- bloc


m <- glm(y ~ x1 + x2 + x3 + x4 + bloc , data=d, family = binomial)
m <- glm(y ~ I(x1^2) + x2 + x3 + x4 + bloc , data=d, family = binomial)
m <- glm(y ~ x1 + I(x1^2) + x2 + x3 + x4 + bloc , data=d, family = binomial)
m <- glm(y ~ x1 + I(x1^2) + log(x2) + x3 + x4  + bloc, data=d, family = binomial)


m <- glm(y ~ x1 + x2 + x3 + x4 + bloc , data=d, family = binomial)
diagplot(m)
diagplot2(m, span = 1/5)


m <- glm(y ~ I(x1^2) + x2 + x3 + x4 + bloc , data=d, family = binomial)
diagplot(m)
diagplot2(m, span = 1/5)


m <- glm(y ~ x1 + I(x1^2) + x2 + x3 + x4 + bloc , data=d, family = binomial)
diagplot(m)
diagplot2(m, span = 1/5)


par(mfrow = c(2,2), mar = c(3,3,0.5,0.5), mgp = c(1.7, 0.6,0))
visreg(m, scale = 'response', partial = TRUE)


#' With binomial models the results on one variable can be strongly influenced by the 
#' chosen value of the other variables even when you don't have interactions
#' 
par(mfrow = c(2,2), mar = c(3,3,2,0.5), mgp = c(1.7, 0.6,0))
visreg(m, xvar = "x1", cond = list(x2 = 0), scale = 'response', partial = TRUE)
title("x2 = 0")
visreg(m, xvar = "x1", cond = list(x2 = 2000), scale = 'response', partial = TRUE)
title("x2 = 2000")
visreg(m, xvar = "x1", cond = list(x2 = 6000), scale = 'response', partial = TRUE)
title("x2 = 6000")
visreg(m, xvar = "x1", cond = list(x2 = 10000), scale = 'response', partial = TRUE)
title("x2 = 10000")












